// Lycée PE MARTIN
// Le 21/1/2023
// Titre : Thermomètre
// Matériels : Arduino Uno, NTC KY-013,
// 0,96" 128x64 OLED 2864 Display module - SSD1306
// Description : voir le sujet
// Ressources
// Afficheur : https://webge.fr/dokuwiki/doku.php?id=materiels:afficheurs:ard0_96
// Bibliothèque Adafruit GFX Graphics Library : https://webge.fr/dokuwiki/doku.php?id=materiels:afficheurs:graphique

// Bibliothèques pour l'afficheur
#include "Wire.h"
#include "Adafruit_GFX.h"
#include "OakOLED.h"

// LED
#define LED 8

// CTN
const float BETA = 3950;

// Variables
float Temp_1 = 0.0;  // Pour la stabilité de l'affichage

// Afficheur
OakOLED oled;  // Construction d'un objet OakOLED

void setup() {
  // Initialisation de la console
  Serial.begin(9600);

  // Configuration de la broche LED
  pinMode(LED, OUTPUT);

  // Initialisation de l'afficheur graphique
  oled.begin();
  oled.setTextSize(1);
  oled.setTextColor(1);
  // Préparation de l'affichage dans le buffer
  oled.setCursor(20, 0);
  oled.println("Thermometre");
  oled.setCursor(0, 15);
  oled.print("Temp: ");
  // Transfert vers l'afficheur
  oled.display();
}


void loop() {
  // Lire
  // ------------------------------
  int N = analogRead(A0);

  // Traiter
  // ------------------------------
  // Modèle linéarisé
  //float Temp = (798.72- N)/11.469;

  // Modèle non linéarisé
  float Temp = 1 / (log(1 / (1023. / N - 1)) / BETA + 1.0 / 298.15) - 273.15;

  // Alarme
  if ((Temp < 24) || (Temp > 30)) {
    digitalWrite(LED, HIGH);
  } else {
    digitalWrite(LED, LOW);
  }

  // Ecrire
  // ------------------------------
  // Ecrire dans la console
  Serial.print("Temperature: ");
  Serial.print(Temp);
  Serial.println("℃");

  // Ecrire sur l'afficheur graphique
  if (Temp_1 != Temp) {
    // Effacement de la zone Temp dans le buffer
    oled.fillRect(36,15,36,7,0); 
    // Affichage de Temp
    oled.setCursor(36,15); 
    oled.print(Temp);
    oled.print("dC");
    oled.display(); // Transfert du buffer dans l'afficheur
    Temp_1 = Temp; 
  }

  // Une mesure par seconde
  delay(1000);
}
